@@ -26,6 +26,7 @@ from logging import DEBUG as _DEBUG
 from logging import INFO as _INFO
 from logging import getLogger
 from math import sqrt as _sqrt
+from struct import unpack as _unpack
 
 import evdev
 import keysyms.keysymdef as _keysymdef
@@ -38,7 +39,6 @@ from yaml import dump_all as _yaml_dump_all
 from yaml import safe_load_all as _yaml_safe_load_all
 
 from .common import NamedInt
-from .common import unpack as _unpack
 from .hidpp20 import FEATURE as _F
 from .special_keys import CONTROL as _CONTROL
 
@@ -133,7 +133,7 @@ def x11_setup():
         if _log.isEnabledFor(_INFO):
             _log.info('X11 library loaded and display set up')
     except Exception:
-        _log.warn('X11 not available - some rule capabilities inoperable: %s', exc_info=_sys.exc_info())
+        _log.warn('X11 not available - some rule capabilities inoperable', exc_info=_sys.exc_info())
         _x11 = False
         xtest_available = False
     return _x11
@@ -151,7 +151,7 @@ def xkb_setup():
         if _log.isEnabledFor(_INFO):
             _log.info('XKB display set up')
     except Exception:
-        _log.warn('XKB display not available - rules cannot access keyboard group: %s', exc_info=_sys.exc_info())
+        _log.warn('XKB display not available - rules cannot access keyboard group', exc_info=_sys.exc_info())
         Xkbdisplay = False
     return Xkbdisplay
 
@@ -174,12 +174,7 @@ key_events = [c for n, c in evdev.ecodes.ecodes.items() if n.startswith('KEY') a
 for (_, evcode) in buttons.values():
     if evcode:
         key_events.append(evcode)
-devicecap = {
-    evdev.ecodes.EV_KEY:
-    key_events,
-    evdev.ecodes.EV_REL:
-    [evdev.ecodes.REL_WHEEL, evdev.ecodes.REL_HWHEEL, evdev.ecodes.REL_WHEEL_HI_RES, evdev.ecodes.REL_HWHEEL_HI_RES]
-}
+devicecap = {evdev.ecodes.EV_KEY: key_events, evdev.ecodes.EV_REL: [evdev.ecodes.REL_WHEEL, evdev.ecodes.REL_HWHEEL]}
 udevice = None
 
 
@@ -191,6 +186,7 @@ def setup_uinput():
         udevice = evdev.uinput.UInput(events=devicecap, name='solaar-keyboard')
         if _log.isEnabledFor(_INFO):
             _log.info('uinput device set up')
+        return True
     except Exception as e:
         _log.warn('cannot create uinput device: %s', e)
 
@@ -824,60 +820,97 @@ class Action(RuleComponent):
 
 
 class KeyPress(Action):
-    def __init__(self, keys):
-        if isinstance(keys, str):
-            keys = [keys]
-        self.key_names = keys
-        self.key_symbols = [XK_KEYS.get(k, None) for k in keys]
+    CLICK, DEPRESS, RELEASE = 'click', 'depress', 'release'
+
+    def __init__(self, args):
+        self.key_names, self.action = self.regularize_args(args)
+        if not isinstance(self.key_names, list):
+            _log.warn('rule KeyPress keys not key names %s', self.keys_names)
+            self.key_symbols = []
+        else:
+            self.key_symbols = [XK_KEYS.get(k, None) for k in self.key_names]
         if not all(self.key_symbols):
-            _log.warn('rule KeyPress not sequence of key names %s', keys)
+            _log.warn('rule KeyPress keys not key names %s', self.key_names)
             self.key_symbols = []
 
+    def regularize_args(self, args):
+        action = self.CLICK
+        if not isinstance(args, list):
+            args = [args]
+        keys = args
+        if len(args) == 2 and args[1] in [self.CLICK, self.DEPRESS, self.RELEASE]:
+            keys = [args[0]] if isinstance(args[0], str) else args[0]
+            action = args[1]
+        return keys, action
+
+    # WARNING:  This is an attempt to reverse the keycode to keysym mappping in XKB.  It may not be completely general.
     def keysym_to_keycode(self, keysym, modifiers):  # maybe should take shift into account
         group = kbdgroup() or 0
         keycodes = gkeymap.get_entries_for_keyval(keysym)
-        if len(keycodes.keys) == 1:
-            k = keycodes.keys[0]
-            return k.keycode
-        else:
-            for k in keycodes.keys:
-                if group is None or group == k.group:
-                    return k.keycode
-            _log.warn('rule KeyPress key symbol not currently available %s', self)
+        (keycode, level) = (None, None)
+        for k in keycodes.keys:  # mappings that have the correct group
+            if group == k.group and k.keycode < 256 and (level is None or k.level < level):
+                keycode = k.keycode
+                level = k.level
+        if keycode or group == 0:
+            return (keycode, level)
+        for k in keycodes.keys:  # mappings for group 0 where keycode only has group 0 mappings
+            if 0 == k.group and k.keycode < 256 and (level is None or k.level < level):
+                (a, m, vs) = gkeymap.get_entries_for_keycode(k.keycode)
+                if a and all(mk.group == 0 for mk in m):
+                    keycode = k.keycode
+                    level = k.level
+        return (keycode, level)
 
     def __str__(self):
-        return 'KeyPress: ' + ' '.join(self.key_names)
+        return 'KeyPress: ' + ' '.join(self.key_names) + ' ' + self.action
 
     def needed(self, k, modifiers):
         code = modifier_code(k)
         return not (code is not None and modifiers & (1 << code))
 
+    def mods(self, level, modifiers, direction):
+        if level == 2 or level == 3:
+            (sk, _) = self.keysym_to_keycode(XK_KEYS.get('ISO_Level3_Shift', None), modifiers)
+            if sk and self.needed(sk, modifiers):
+                simulate_key(sk, direction)
+        if level == 1 or level == 3:
+            (sk, _) = self.keysym_to_keycode(XK_KEYS.get('Shift_L', None), modifiers)
+            if sk and self.needed(sk, modifiers):
+                simulate_key(sk, direction)
+
     def keyDown(self, keysyms, modifiers):
         for k in keysyms:
-            keycode = self.keysym_to_keycode(k, modifiers)
-            if keycode and self.needed(keycode, modifiers):
+            (keycode, level) = self.keysym_to_keycode(k, modifiers)
+            if keycode is None:
+                _log.warn('rule KeyPress key symbol not currently available %s', self)
+            elif self.needed(keycode, modifiers):
+                self.mods(level, modifiers, _KEY_PRESS)
                 simulate_key(keycode, _KEY_PRESS)
 
     def keyUp(self, keysyms, modifiers):
         for k in keysyms:
-            keycode = self.keysym_to_keycode(k, modifiers)
+            (keycode, level) = self.keysym_to_keycode(k, modifiers)
             if keycode and self.needed(keycode, modifiers):
                 simulate_key(keycode, _KEY_RELEASE)
+                self.mods(level, modifiers, _KEY_RELEASE)
 
     def evaluate(self, feature, notification, device, status, last_result):
         if gkeymap:
             current = gkeymap.get_modifier_state()
             if _log.isEnabledFor(_INFO):
-                _log.info('KeyPress action: %s, modifiers %s %s', self.key_names, current, [hex(k) for k in self.key_symbols])
-            self.keyDown(self.key_symbols, current)
-            self.keyUp(reversed(self.key_symbols), current)
+                _log.info('KeyPress action: %s %s, group %s, modifiers %s', self.key_names, self.action, kbdgroup(), current)
+            if self.action != self.RELEASE:
+                self.keyDown(self.key_symbols, current)
+            if self.action != self.DEPRESS:
+                self.keyUp(reversed(self.key_symbols), current)
             _time.sleep(0.01)
         else:
             _log.warn('no keymap so cannot determine which keycode to send')
         return None
 
     def data(self):
-        return {'KeyPress': [str(k) for k in self.key_names]}
+        return {'KeyPress': [[str(k) for k in self.key_names], self.action]}
 
 
 # KeyDown is dangerous as the key can auto-repeat and make your system unusable
