@@ -20,6 +20,8 @@ from logging import DEBUG as _DEBUG
 from logging import INFO as _INFO
 from logging import WARN as _WARN
 from logging import getLogger
+from struct import pack as _pack
+from struct import unpack as _unpack
 from time import time as _time
 
 from . import hidpp10 as _hidpp10
@@ -29,7 +31,6 @@ from .common import NamedInt as _NamedInt
 from .common import NamedInts as _NamedInts
 from .common import bytes2int as _bytes2int
 from .common import int2bytes as _int2bytes
-from .common import unpack as _unpack
 from .i18n import _
 from .settings import ActionSettingRW as _ActionSettingRW
 from .settings import BitFieldSetting as _BitFieldSetting
@@ -41,6 +42,8 @@ from .settings import ChoicesValidator as _ChoicesV
 from .settings import FeatureRW as _FeatureRW
 from .settings import LongSettings as _LongSettings
 from .settings import MultipleRangeValidator as _MultipleRangeV
+from .settings import PackedRangeValidator as _PackedRangeV
+from .settings import RangeFieldSetting as _RangeFieldSetting
 from .settings import RangeValidator as _RangeV
 from .settings import Setting as _Setting
 from .settings import Settings as _Settings
@@ -75,6 +78,7 @@ _GP = _hidpp20.PARAM
 #    and permit reading or writing the entire map or just one key/value pair.
 # The _BitFieldSetting class is for settings that have multiple boolean values packed into a bit field.
 # _BitFieldOMSetting is similar.
+# The _RangeFieldSetting class is for settings that have multiple ranges packed into a byte string.
 # _LongSettings is for settings that have an even more complex structure.
 #
 # When settings are created a reader/writer and a validator are created.
@@ -755,7 +759,6 @@ class MouseGesture(_Setting):
             if self.fsmState == 'pressed':
                 # emit mouse gesture notification
                 from .base import _HIDPP_Notification as _HIDPP_Notification
-                from .common import pack as _pack
                 from .diversion import process_notification as _process_notification
                 self.push_mouse_event()
                 if _log.isEnabledFor(_INFO):
@@ -1140,10 +1143,61 @@ class PersistentRemappableAction(_Settings):
                 return None
             choices = {}
             for k in remap_keys:
-                key = _special_keys.CONTROL[k.key]
-                choices[key] = keys  # TO RECOVER FROM BAD VALUES use _special_keys.KEYS
+                if k is not None:
+                    key = _special_keys.CONTROL[k.key]
+                    choices[key] = keys  # TO RECOVER FROM BAD VALUES use _special_keys.KEYS
             return cls(choices, key_byte_count=2, byte_count=4) if choices else None
 
+        def validate_read(self, reply_bytes, key):
+            start = self._key_byte_count + self._read_skip_byte_count
+            end = start + self._byte_count
+            reply_value = _bytes2int(reply_bytes[start:end]) & self.mask
+            # Craft keyboard has a value that isn't valid so fudge these values
+            if reply_value not in self.choices[key]:
+                if _log.isEnabledFor(_WARN):
+                    _log.warn('unusual persistent remappable action mapping %x: use Default', reply_value)
+                reply_value = _special_keys.KEYS_Default
+            return reply_value
+
+
+class Sidetone(_Setting):
+    name = 'sidetone'
+    label = _('Sidetone')
+    description = _('Set sidetone level.')
+    feature = _F.SIDETONE
+    validator_class = _RangeV
+    min_value = 0
+    max_value = 100
+
+
+class Equalizer(_RangeFieldSetting):
+    name = 'equalizer'
+    label = _('Equalizer')
+    description = _('Set equalizer levels.')
+    feature = _F.EQUALIZER
+    rw_options = {'read_fnid': 0x20, 'write_fnid': 0x30, 'read_prefix': b'\x00'}
+    keys_universe = []
+
+    class validator_class(_PackedRangeV):
+        @classmethod
+        def build(cls, setting_class, device):
+            data = device.feature_request(_F.EQUALIZER, 0x00)
+            if not data:
+                return None
+            count, dbRange, _x, dbMin, dbMax = _unpack('!BBBBB', data[:5])
+            if dbMin == 0:
+                dbMin = -dbRange
+            if dbMax == 0:
+                dbMax = dbRange
+            map = _NamedInts()
+            for g in range((count + 6) // 7):
+                freqs = device.feature_request(_F.EQUALIZER, 0x10, g * 7)
+                for b in range(7):
+                    if g * 7 + b >= count:
+                        break
+                    map[g * 7 + b] = str(int.from_bytes(freqs[2 * b + 1:2 * b + 3], 'big')) + _('Hz')
+            return cls(map, min_value=dbMin, max_value=dbMax, count=count, write_prefix_bytes=b'\x02')
+
 
 SETTINGS = [
     RegisterHandDetection,  # simple
@@ -1158,8 +1212,8 @@ SETTINGS = [
     HiresMode,  # simple
     SmartShift,  # working
     SmartShiftEnhanced,  # simple
-    ThumbMode,  # working
     ThumbInvert,  # working
+    ThumbMode,  # working
     OnboardProfiles,
     ReportRate,  # working
     PointerSpeed,  # simple
@@ -1177,8 +1231,8 @@ SETTINGS = [
     PersistentRemappableAction,
     DivertKeys,  # working
     DisableKeyboardKeys,  # working
-    DivertCrown,  # working
     CrownSmooth,  # working
+    DivertCrown,  # working
     DivertGkeys,  # working
     MKeyLEDs,  # working
     MRKeyLED,  # working
@@ -1188,6 +1242,8 @@ SETTINGS = [
     Gesture2Gestures,  # working
     Gesture2Divert,
     Gesture2Params,  # working
+    Sidetone,
+    Equalizer,
 ]
 
 #
@@ -1219,15 +1275,15 @@ def check_feature_settings(device, already_known):
     newAbsent = []
     for sclass in SETTINGS:
         if sclass.feature:
-            if sclass.name not in absent and not any(s.name == sclass.name for s in already_known):
+            known_present = device.persister and sclass.name in device.persister
+            if not any(s.name == sclass.name for s in already_known) and (known_present or sclass.name not in absent):
                 setting = check_feature(device, sclass)
                 if setting:
                     already_known.append(setting)
                     if sclass.name in newAbsent:
                         newAbsent.remove(sclass.name)
-                else:
-                    if not any(s.name == sclass.name for s in already_known) and sclass.name not in newAbsent:
-                        newAbsent.append(sclass.name)
+                elif sclass.name not in newAbsent and sclass.name not in absent and sclass.name not in device.persister:
+                    newAbsent.append(sclass.name)
     if device.persister and newAbsent:
         absent.extend(newAbsent)
         device.persister['_absent'] = absent
