@@ -1,5 +1,4 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2012-2013  Daniel Pavel
 ##
@@ -17,8 +16,6 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 import argparse as _argparse
 import sys as _sys
 
@@ -165,6 +162,7 @@ def _find_device(receivers, name):
                 if dev:
                     yield dev
         else:  # wired device, make a device list from it
+            r.ping()
             r = [r]
 
         for dev in r:
